#include <stdint.h>
#include <string.h>


#define cb_item_t	uint8_t		// item type

#define CB_ITEM_N	1024		// item number，实际可存入数据少一个，留一个空位表示满，以区别于空


typedef struct
{
    cb_item_t buf[CB_ITEM_N];
    int       wrptr;            // 写指针
    int       rdptr;            // 读指针
} CircleBuffer_t;


/* Element Count in buffer */
static int CirBuf_Count(CircleBuffer_t *cirbuf)
{
    return (cirbuf->wrptr - cirbuf->rdptr) & (CB_ITEM_N - 1);
}

/* Space available in buffer */
static int CirBuf_Space(CircleBuffer_t *cirbuf)
{
    return (cirbuf->rdptr - cirbuf->wrptr - 1) & (CB_ITEM_N - 1);
}

static int CirBuf_Empty(CircleBuffer_t *cirbuf)
{
    return CirBuf_Count(cirbuf) == 0;
}

static int CirBuf_Full(CircleBuffer_t *cirbuf)
{
    return CirBuf_Space(cirbuf) == 0;
}

static void CirBuf_Clear(CircleBuffer_t *cirbuf)
{
    cirbuf->rdptr = cirbuf->wrptr;
}

/**
 * @brief 向环形缓冲区中写入数据 
 * @param cirbuf 环形缓冲区结构体指针 
 * @param buf    源数据缓冲区指针 
 * @param num    请求写入的数据项数量 
 * @return int   实际写入的数据项数量 
 */
static int CirBuf_Write(CircleBuffer_t *cirbuf, cb_item_t *buf, int num) {
    int n; // 临时变量，记录当前可写入的空间大小 
 
    // 情况1：写指针(wrptr)在读指针(rdptr)之后或相等（未绕环）
    if (cirbuf->wrptr >= cirbuf->rdptr) {
        // 计算从写指针到缓冲区末尾的剩余空间（CB_ITEM_N - wrptr）
        n = CB_ITEM_N - cirbuf->wrptr;
 
        // 特殊情况：如果读指针在0位置，需保留一个空位以避免与读指针重合（判断缓冲区满的条件）
        if (cirbuf->rdptr == 0)
            n -= 1; // 减少一个可写入位置 
 
        // 取最小值：避免写入数量超过当前段剩余空间 
        n = (n >= num) ? num : n;
 
        // 将数据从源缓冲区拷贝到环形缓冲区 
        memcpy(&cirbuf->buf[cirbuf->wrptr], buf, n * sizeof(cb_item_t));
 
        // 更新写指针（考虑绕环）
        cirbuf->wrptr = (cirbuf->wrptr + n) % CB_ITEM_N;
 
        // 终止条件：已写入足够数量或缓冲区已满 
        if ((n == num) || CirBuf_Full(cirbuf)) {
            return n;
        }
        // 递归写入剩余数据（从缓冲区头部开始）
        else {
            return n + CirBuf_Write(cirbuf, &buf[n], num - n);
        }
    }
    // 情况2：写指针(wrptr)在读指针(rdptr)之前（已绕环）
    else {
        // 计算剩余可写入空间（rdptr - wrptr - 1）
        n = CirBuf_Space(cirbuf);
 
        // 取最小值：避免写入数量超过剩余空间 
        n = (n >= num) ? num : n;
 
        // 拷贝数据到环形缓冲区 
        memcpy(&cirbuf->buf[cirbuf->wrptr], buf, n * sizeof(cb_item_t));
 
        // 更新写指针（线性增加，无需绕环）
        cirbuf->wrptr += n;
 
        return n; // 返回实际写入数量 
    }
}

/**
 * @brief 从环形缓冲区中读取数据 
 * @param cirbuf 环形缓冲区结构体指针 
 * @param buf    目标存储缓冲区指针 
 * @param num    请求读取的数据项数量 
 * @return int   实际读取的数据项数量 
 */
static int CirBuf_Read(CircleBuffer_t *cirbuf, cb_item_t *buf, int num) {
    int n; // 临时变量，记录当前可读取的数据量 
 
    // 情况1：写指针(wrptr)在读指针(rdptr)之后或相等（未绕环）
    if (cirbuf->wrptr >= cirbuf->rdptr) {
        // 计算可读取的数据项数量（wrptr - rdptr）
        n = CirBuf_Count(cirbuf);
 
        // 取最小值：避免请求数量超过实际可读数量 
        n = (n >= num) ? num : n;
 
        // 从环形缓冲区拷贝数据到目标缓冲区 
        memcpy(buf, &cirbuf->buf[cirbuf->rdptr], n * sizeof(cb_item_t));
 
        // 更新读指针（线性增加）
        cirbuf->rdptr += n;
 
        return n; // 返回实际读取数量 
    }
    // 情况2：写指针(wrptr)在读指针(rdptr)之前（已绕环）
    else {
        // 计算从读指针到缓冲区末尾的数据项数量（CB_ITEM_N - rdptr）
        n = CB_ITEM_N - cirbuf->rdptr;
 
        // 取最小值：避免请求数量超过当前段可读数量 
        n = (n >= num) ? num : n;
 
        // 拷贝当前段数据到目标缓冲区 
        memcpy(buf, &cirbuf->buf[cirbuf->rdptr], n * sizeof(cb_item_t));
 
        // 更新读指针（考虑绕环）
        cirbuf->rdptr = (cirbuf->rdptr + n) % CB_ITEM_N;
 
        // 终止条件：已读取足够数量或缓冲区为空 
        if ((n == num) || CirBuf_Empty(cirbuf)) {
            return n;
        } 
        // 递归读取剩余数据（从缓冲区头部开始）
        else {
            return n + CirBuf_Read(cirbuf, &buf[n], num - n);
        }
    }
}
